
/* Name: main.c
 * Project: AVR USB driver for CDC interface on Low-Speed USB
 * Author: Osamu Tamura
 * Creation Date: 2006-05-12
 * Tabsize: 4
 * Copyright: (c) 2006 by Recursion Co., Ltd.
 * License: Proprietary, free under certain conditions. See Documentation.
 *
 * 2006-07-08   removed zero-sized receive block
 * 2006-07-08   adapted to higher baud rate by T.Kitazawa
 *
 */

#include "usbconfig.h"

#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <util/delay.h>

#include "osccal.h"
#include "usbdrv.h"

#define HW_CDC_BULK_OUT_SIZE     8
#define HW_CDC_BULK_IN_SIZE      8
/* Size of bulk transfer packets. The standard demands 8 bytes, but we may
 * be better off with less. Try smaller values if the communication hangs.
 */

#define NUM_COLS 3
static uint8_t gColsR[NUM_COLS];
static uint8_t gColsG[NUM_COLS];
static uint8_t gColsB[NUM_COLS];


#define UART_DEFAULT_BPS     4800


enum {
    SEND_ENCAPSULATED_COMMAND = 0,
    GET_ENCAPSULATED_RESPONSE,
    SET_COMM_FEATURE,
    GET_COMM_FEATURE,
    CLEAR_COMM_FEATURE,
    SET_LINE_CODING = 0x20,
    GET_LINE_CODING,
    SET_CONTROL_LINE_STATE,
    SEND_BREAK
};


static PROGMEM char configDescrCDC[] = {   /* USB configuration descriptor */
    9,          /* sizeof(usbDescrConfig): length of descriptor in bytes */
    USBDESCR_CONFIG,    /* descriptor type */
    67,
    0,          /* total length of data returned (including inlined descriptors) */
    2,          /* number of interfaces in this configuration */
    1,          /* index of this configuration */
    0,          /* configuration name string index */
#if USB_CFG_IS_SELF_POWERED
    (1 << 7) | USBATTR_SELFPOWER,       /* attributes */
#else
    (1 << 7),                           /* attributes */
#endif
    USB_CFG_MAX_BUS_POWER/2,            /* max USB current in 2mA units */

    /* interface descriptor follows inline: */
    9,          /* sizeof(usbDescrInterface): length of descriptor in bytes */
    USBDESCR_INTERFACE, /* descriptor type */
    0,          /* index of this interface */
    0,          /* alternate setting for this interface */
    USB_CFG_HAVE_INTRIN_ENDPOINT,   /* endpoints excl 0: number of endpoint descriptors to follow */
    USB_CFG_INTERFACE_CLASS,
    USB_CFG_INTERFACE_SUBCLASS,
    USB_CFG_INTERFACE_PROTOCOL,
    0,          /* string index for interface */

    /* CDC Class-Specific descriptor */
    5,           /* sizeof(usbDescrCDC_HeaderFn): length of descriptor in bytes */
    0x24,        /* descriptor type */
    0,           /* header functional descriptor */
    0x10, 0x01,

    4,           /* sizeof(usbDescrCDC_AcmFn): length of descriptor in bytes    */
    0x24,        /* descriptor type */
    2,           /* abstract control management functional descriptor */
    0x02,        /* SET_LINE_CODING, GET_LINE_CODING, SET_CONTROL_LINE_STATE    */

    5,           /* sizeof(usbDescrCDC_UnionFn): length of descriptor in bytes  */
    0x24,        /* descriptor type */
    6,           /* union functional descriptor */
    0,           /* CDC_COMM_INTF_ID */
    1,           /* CDC_DATA_INTF_ID */

    5,           /* sizeof(usbDescrCDC_CallMgtFn): length of descriptor in bytes */
    0x24,        /* descriptor type */
    1,           /* call management functional descriptor */
    3,           /* allow management on data interface, handles call management by itself */
    1,           /* CDC_DATA_INTF_ID */

    /* Endpoint Descriptor */
    7,           /* sizeof(usbDescrEndpoint) */
    USBDESCR_ENDPOINT,  /* descriptor type = endpoint */
    0x80|USB_CFG_EP3_NUMBER,        /* IN endpoint number 3 */
    0x03,        /* attrib: Interrupt endpoint */
    8, 0,        /* maximum packet size */
    USB_CFG_INTR_POLL_INTERVAL,        /* in ms */

    /* Interface Descriptor  */
    9,           /* sizeof(usbDescrInterface): length of descriptor in bytes */
    USBDESCR_INTERFACE,           /* descriptor type */
    1,           /* index of this interface */
    0,           /* alternate setting for this interface */
    2,           /* endpoints excl 0: number of endpoint descriptors to follow */
    0x0A,        /* Data Interface Class Codes */
    0,
    0,           /* Data Interface Class Protocol Codes */
    0,           /* string index for interface */

    /* Endpoint Descriptor */
    7,           /* sizeof(usbDescrEndpoint) */
    USBDESCR_ENDPOINT,  /* descriptor type = endpoint */
    0x01,        /* OUT endpoint number 1 */
    0x02,        /* attrib: Bulk endpoint */
    HW_CDC_BULK_OUT_SIZE, 0,        /* maximum packet size */
    0,           /* in ms */

    /* Endpoint Descriptor */
    7,           /* sizeof(usbDescrEndpoint) */
    USBDESCR_ENDPOINT,  /* descriptor type = endpoint */
    0x81,        /* IN endpoint number 1 */
    0x02,        /* attrib: Bulk endpoint */
    HW_CDC_BULK_IN_SIZE, 0,        /* maximum packet size */
    0,           /* in ms */
};


uchar usbFunctionDescriptor(usbRequest_t *rq)
{

    if(rq->wValue.bytes[1] == USBDESCR_DEVICE){
        usbMsgPtr = (uchar *)usbDescriptorDevice;
        return usbDescriptorDevice[0];
    }else{  /* must be config descriptor */
        usbMsgPtr = (uchar *)configDescrCDC;
        return sizeof(configDescrCDC);
    }
}


uchar               sendEmptyFrame;
static uchar        intr3Status;    /* used to control interrupt endpoint transmissions */

static usbWord_t    baud;


/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

uchar usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;

    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* class request type */

        if( rq->bRequest==GET_LINE_CODING || rq->bRequest==SET_LINE_CODING ){
            return 0xff;
        /*    GET_LINE_CODING -> usbFunctionRead()    */
        /*    SET_LINE_CODING -> usbFunctionWrite()    */
        }
        if(rq->bRequest == SET_CONTROL_LINE_STATE){
            /* Report serial state (carrier detect). On several Unix platforms,
             * tty devices can only be opened when carrier detect is set.
             */
            if( intr3Status==0 )
                intr3Status = 2;
        }

        /*  Prepare bulk-in endpoint to respond to early termination   */
        if((rq->bmRequestType & USBRQ_DIR_MASK) == USBRQ_DIR_HOST_TO_DEVICE)
            sendEmptyFrame  = 1;
    }

    return 0;
}


/*---------------------------------------------------------------------------*/
/* usbFunctionRead                                                          */
/*---------------------------------------------------------------------------*/

uchar usbFunctionRead( uchar *data, uchar len )
{

    data[0] = baud.bytes[0];
    data[1] = baud.bytes[1];
    data[2] = 0;
    data[3] = 0;
    data[4] = 0;
    data[5] = 0;
    data[6] = 8;

    return 7;
}


/*---------------------------------------------------------------------------*/
/* usbFunctionWrite                                                          */
/*---------------------------------------------------------------------------*/

uchar usbFunctionWrite( uchar *data, uchar len )
{

    /*    SET_LINE_CODING    */
    baud.bytes[0] = data[0];
    baud.bytes[1] = data[1];

    //TODO: new baud rate is baud.word;

    return 1;
}


uint8_t b_dat[8];	//#RRGGBB\n or #RRGGBB\r -> 8 chars anyways
uint8_t b_len = 0;


static inline uint8_t hexch(uint8_t v){

	if (v >= '0' && v <= '9')
		return v - '0';
	if (v >= 'A' && v <= 'F')
		return v + 10 -  'A';
	if (v >= 'a' && v <= 'f')
		return v + 10 -  'a';

	return 0xff;
}

void usbFunctionWriteOut( uchar *data, uchar len )
{
    uint8_t i;
    uint8_t rl, rh,gl,gh,bl,bh, t;
    while(len--){
    	//shift all left
    	for(i = 0; i < sizeof(b_dat) - 1; i++) b_dat[i] = b_dat[i + 1];
    	
    	//apend new byte
    	b_dat[sizeof(b_dat) - 1] = *data++;
    	
    	if(b_dat[0] > '0' + NUM_COLS - 1 || b_dat[0] < '0') continue;
    	rh = hexch(b_dat[1]); if(rh == 0xff) continue;
    	rl = hexch(b_dat[2]); if(rl == 0xff) continue;
    	gh = hexch(b_dat[3]); if(gh == 0xff) continue;
    	gl = hexch(b_dat[4]); if(gl == 0xff) continue;
    	bh = hexch(b_dat[5]); if(bh == 0xff) continue;
    	bl = hexch(b_dat[6]); if(bl == 0xff) continue;
    	if(b_dat[7] != '\r' && b_dat[7] != '\n') continue;
    	rh <<= 4;
    	gh <<= 4;
    	bh <<= 4;
    	rh |= rl;
    	gh |= gl;
    	bh |= bl;
    	t = b_dat[0] - '0';
    	
    	gColsR[t] = rh;
    	gColsG[t] = gh;
    	gColsB[t] = bh;
    }
}


static void hardwareInit(void)
{

    /* activate pull-ups except on USB lines */
    USB_CFG_IOPORT   = (uchar)~((1<<USB_CFG_DMINUS_BIT)|(1<<USB_CFG_DPLUS_BIT));
    /* all pins input except USB (-> USB reset) */
#ifdef USB_CFG_PULLUP_IOPORT    /* use usbDeviceConnect()/usbDeviceDisconnect() if available */
    USBDDR    = 0;    /* we do RESET by deactivating pullup */
    usbDeviceDisconnect();
#else
    USBDDR    = (1<<USB_CFG_DMINUS_BIT)|(1<<USB_CFG_DPLUS_BIT);
#endif

    /* 250 ms disconnect */
    wdt_reset();
    _delay_ms(250);

#ifdef USB_CFG_PULLUP_IOPORT
    usbDeviceConnect();
#else
    USBDDR    = 0;      /*  remove USB reset condition */
#endif

    /*    USART configuration    */
    baud.word  = UART_DEFAULT_BPS;
}

void    usbEventResetReady(void)
{
    /* Disable interrupts during oscillator calibration since
     * usbMeasureFrameLength() counts CPU cycles.
     */
    cli();
    calibrateOscillator();
    sei();
}

int main(void)
{
    wdt_disable();

#define PORT_RED_0	PORTA
#define PORT_GRN_0	PORTA
#define PORT_BLU_0	PORTA
#define PIN_RED_0	PINA
#define PIN_GRN_0	PINA
#define PIN_BLU_0	PINA
#define BIT_RED_0	(1 << 0)
#define BIT_GRN_0	(1 << 1)
#define BIT_BLU_0	(1 << 2)

#define PORT_RED_1	PORTA
#define PORT_GRN_1	PORTA
#define PORT_BLU_1	PORTA
#define PIN_RED_1	PINA
#define PIN_GRN_1	PINA
#define PIN_BLU_1	PINA
#define BIT_RED_1	(1 << 5)
#define BIT_GRN_1	(1 << 6)
#define BIT_BLU_1	(1 << 7)

#define PORT_RED_2	PORTB
#define PORT_GRN_2	PORTB
#define PORT_BLU_2	PORTB
#define PIN_RED_2	PINB
#define PIN_GRN_2	PINB
#define PIN_BLU_2	PINB
#define BIT_RED_2	(1 << 4)
#define BIT_GRN_2	(1 << 5)
#define BIT_BLU_2	(1 << 6)


    hardwareInit();
    usbInit();
    
    DDRB |= 0x70;	//RGB2
    DDRA |= 0xE7;	//RGB1, RGB0
    PORTB = 0x70;	//set all my pins high
    PORTA = 0xE7;

    intr3Status = 0;
    sendEmptyFrame  = 0;

    sei();
    for(;;){    /* main event loop */
        wdt_reset();
        usbPoll();
        
        //DO PWM (approximately even - some jitter may exist)
        {
        	static uint8_t i = 0;
        	

		#define CASE_N(_case, _idx, _col, __shortcol)									\
				PORT_ ## _col ## _ ## _idx |= BIT_ ## _col ## _ ## _idx;					\
				if (i < gCols ## __shortcol [_idx]) PIN_ ## _col ## _ ## _idx = BIT_ ## _col ## _ ## _idx
		
		#define LED_N(_led)											\
			CASE_N(_led*3+0,_led,RED,R);									\
			CASE_N(_led*3+1,_led,GRN,G);									\
			CASE_N(_led*3+2,_led,BLU,B);
			
			
			LED_N(0);
			LED_N(1);
			LED_N(2);
	
        	
        	i++;
        }
        
   //we do not send data in this project
   #define send_sz __xxx
   {
       uint8_t __xxx = 0;


        //send data to PC if needed (b_len is len available (up to 128), rx_buf is buffer
        if( usbInterruptIsReady() && (send_sz||sendEmptyFrame) ) {
        	
            usbSetInterrupt(b_dat, send_sz);
            sendEmptyFrame    = send_sz & HW_CDC_BULK_IN_SIZE;
            send_sz    = 0;
        }
    }
    #undef send_sz
   ///end above comment's effect
        
   //we always have space for host to give us dat ain this project
   #define b_len 0
        //let host give us more if we have bufer space for HW_CDC_BULK_OUT_SIZE more bytes
        if( usbAllRequestsAreDisabled() && !b_len) {
            usbEnableAllRequests();
        }
   #undef b_len
   ///end above comment's effect

        /* We need to report rx and tx carrier after open attempt */
        if(intr3Status != 0 && usbInterruptIsReady3()){
            static uchar serialStateNotification[10] = {0xa1, 0x20, 0, 0, 0, 0, 2, 0, 3, 0};

            if(intr3Status == 2){
                usbSetInterrupt3(serialStateNotification, 8);
            }else{
                usbSetInterrupt3(serialStateNotification+8, 2);
            }
            intr3Status--;
        }
    }
    return 0;
}

